BASE_URL = "https://networkrepository.com/"

HEADERS = {
    "User-Agent": (
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/130.0.0.0 Safari/537.36"
    ),
    "Accept": "text/plain, */*; q=0.01",
    "X-Requested-With": "XMLHttpRequest",
    "Origin": "https://networkrepository.com",
    "Referer": "https://networkrepository.com/",
}

COLLECTIONS = {
    "asn",  # Animal Social Networks
    "bio",  # Biological Networks
    "bn",  # Brain Networks
    "ca",  # Collaboration Networks
    "cit",  # Citation Networks
    "dimacs",  # DIMACS Networks
    "dimacs10",  # DIMACS10 Networks
    "dynamic",  # Dynamic Networks
    "eco",  # Ecology Networks
    "econ",  # Economic Networks
    "email",  # Email Networks
    "graph500",  # Graph500 Networks
    "heter",  # Heterogeneous Networks
    "ia",  # Interaction Networks
    "inf",  # Infrastructure Networks
    "labeled",  # Labeled Networks
    "massive",  # Massive Networks
    "misc",  # Miscellaneous Networks
    "power",  # Power Networks
    "protein",  # Protein Networks
    "proximity",  # Proximity Networks
    "rand",  # Random Networks
    "rec",  # Recommendation Networks
    "rt",  # Retweet Networks
    "road",  # Road Networks
    # Retweet Networks
    "sc",  # Small-World Networks
    "soc",  # Social Networks
    "socfb",  # Facebook Networks
    "tech",  # Technological Networks
    "tscc",  # TSCC Networks
    "web",  # Web Graphs
}
